/***************************************************************************
                          extractdlg.cpp  -  description
                             -------------------
    begin                : Sun Apr 8 2001
    copyright            : (C) 2001 by Javier Campos Morales
    email                : javi_cms@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "extractdlg.h"
#include "extractdlg.moc"

#include <klocale.h>
#include <kstddirs.h>
#include <kconfig.h>
#include <kfiledialog.h>
#include <kmessagebox.h>

#include <qbuttongroup.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qdir.h>

ExtractDlg::ExtractDlg( QWidget *parent, const char *name ) : KDialogBase(Swallow, i18n("Extract options"), Ok|Cancel, Ok, parent, name, true, true)
{
	KConfig *config = KGlobal::config();

	config->setGroup("Extract options");
	QSize *defaultSize = new QSize(400,400);
	this->resize( config->readSizeEntry("Geometry", defaultSize ) );

	initGUI();

	if( config->readBoolEntry( "Compress in dir", false ) )
		kurlrequest->setURL( config->readEntry("Directory", "" ) );
}

ExtractDlg::~ExtractDlg(){
}

/** Init graphical interface */
void ExtractDlg::initGUI()
{
		QWidget *mainWidget = new QWidget(this);

    QVBoxLayout *Form1Layout = new QVBoxLayout( mainWidget );
    Form1Layout->setSpacing( 6 );

    QHBoxLayout *Layout6 = new QHBoxLayout;
    Layout6->setSpacing( 6 );
    Layout6->setMargin( 0 );

    QLabel *PixmapLabel1 = new QLabel( mainWidget, "PixmapLabel1" );
    PixmapLabel1->setMinimumSize( QSize( 48, 48 ) );
    PixmapLabel1->setMaximumSize( QSize( 48, 48 ) );
    PixmapLabel1->setPixmap( QPixmap( locate("icon","hicolor/48x48/mimetypes/tgz.png") ) );
    Layout6->addWidget( PixmapLabel1 );

    QVBoxLayout *Layout5 = new QVBoxLayout;
    Layout5->setSpacing( 6 );
    Layout5->setMargin( 0 );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Preferred );
    Layout5->addItem( spacer );

    QGridLayout *Layout1 = new QGridLayout;
    Layout1->setSpacing( 6 );
    Layout1->setMargin( 0 );

    QLabel *TextLabel2 = new QLabel( mainWidget, "TextLabel2" );
    TextLabel2->setText( i18n( "Name :" ) );
    TextLabel2->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
    Layout1->addWidget( TextLabel2, 0, 0 );

    QLabel *TextLabel3 = new QLabel( mainWidget, "TextLabel3" );
    TextLabel3->setText( i18n( "Directory :" ) );
    TextLabel3->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );
    Layout1->addWidget( TextLabel3, 1, 0 );

    DirName_label = new QLabel( mainWidget, "TextLabel5" );
    DirName_label->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, DirName_label->sizePolicy().hasHeightForWidth() ) );
    Layout1->addWidget( DirName_label, 1, 1 );

    FileName_label = new QLabel( mainWidget, "TextLabel4" );
    FileName_label->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, FileName_label->sizePolicy().hasHeightForWidth() ) );
    Layout1->addWidget( FileName_label, 0, 1 );

    Layout5->addLayout( Layout1 );
    QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Preferred );
    Layout5->addItem( spacer_2 );
    Layout6->addLayout( Layout5 );
    Form1Layout->addLayout( Layout6 );

    view_extractfiles = new QMultiLineEdit( mainWidget, "MultiLineEdit1" );
    Form1Layout->addWidget( view_extractfiles );

    QButtonGroup *ButtonGroup1 = new QButtonGroup( mainWidget, "ButtonGroup1" );
    ButtonGroup1->setTitle( i18n( "Actions" ) );
    ButtonGroup1->setColumnLayout(0, Qt::Vertical );
    ButtonGroup1->layout()->setSpacing( 0 );
    ButtonGroup1->layout()->setMargin( 0 );
    QVBoxLayout *ButtonGroup1Layout = new QVBoxLayout( ButtonGroup1->layout() );
    ButtonGroup1Layout->setAlignment( Qt::AlignTop );
    ButtonGroup1Layout->setSpacing( 6 );
    ButtonGroup1Layout->setMargin( 11 );

    extractCurrent_radio = new QRadioButton( ButtonGroup1, "RadioButton6" );
    extractCurrent_radio->setText( i18n( "Extract files in current Directory." ) );
		connect( extractCurrent_radio, SIGNAL( clicked() ), this, SLOT( slotExtractCheck() ) );
    ButtonGroup1Layout->addWidget( extractCurrent_radio );

    extractDir_radio = new QRadioButton( ButtonGroup1, "RadioButton7" );
    extractDir_radio->setText( i18n( "Extract files in ..." ) );
    extractDir_radio->setChecked(true);
		connect( extractDir_radio, SIGNAL( clicked() ), this, SLOT( slotExtractCheck() ) );
    ButtonGroup1Layout->addWidget( extractDir_radio );

    QHBoxLayout *Layout7 = new QHBoxLayout;
    Layout7->setSpacing( 6 );
    Layout7->setMargin( 0 );
    QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Preferred, QSizePolicy::Minimum );
    Layout7->addItem( spacer_3 );

		kurlrequest = new KURLRequester( ButtonGroup1, "openurl" );
		(kurlrequest->fileDialog())->setMode(  (KFile::Mode)(KFile::Directory|KFile::LocalOnly) );
    Layout7->addWidget( kurlrequest );
    ButtonGroup1Layout->addLayout( Layout7 );

    Form1Layout->addWidget( ButtonGroup1 );

		this->setMainWidget( mainWidget );
}

/** No descriptions */
void ExtractDlg::slotOk()
{
  if( !QDir::QDir( kurlrequest->url() ).exists() )
		{
		KMessageBox::error(0, i18n("SORRY !\n\nDirectoy doesn't exists.\n"), i18n("I cannot do this !"), false);
		return;
		}

	if( ( kurlrequest->url() ).right(1) != "/" ) kurlrequest->setURL( kurlrequest->url() + "/" );

  KConfig *config = KGlobal::config();

	config->setGroup("Extract options");
	config->writeEntry( "Geometry", this->size() );
	config->writeEntry( "Compress in dir", extractDir_radio->isChecked() );
	config->writeEntry( "Directory", kurlrequest->url() );
  config->sync();

	QApplication::disconnect( &q, SIGNAL(receivedStdout(KProcess *, char *, int)),this,SLOT(slotOutputMessages(KProcess *, char *, int)));
	QApplication::disconnect( &q, SIGNAL(receivedStderr(KProcess *, char *, int)),this,SLOT(slotOutputMessages(KProcess *, char *, int)));
	QApplication::connect( &q, SIGNAL(processExited(KProcess *)),this, SLOT(slotFinishAction()) );

	if( extractCurrent_radio->isChecked() )
	{
  if( (FileName_label->text()).right(7) == "tar.bz2" )
		execute((  "bzip2 -cd " + DirName_label->text() + FileName_label->text() + " | tar xvf - -C " + DirName_label->text() ).latin1() );
		else
	  execute((  "tar zxvf " + DirName_label->text() + FileName_label->text() + " -C " + DirName_label->text() ).latin1() );
	}
	else
	{
  if( (FileName_label->text()).right(7) == "tar.bz2" )
		execute((  "bzip2 -cd " + DirName_label->text() + FileName_label->text() + " | tar xvf - -C " + kurlrequest->url() ).latin1() );
		else
	  execute((  "tar zxvf " + DirName_label->text() + FileName_label->text() + " -C " + kurlrequest->url() ).latin1() );
	}

	this->accept();
}

/** Set name of the file */
void ExtractDlg::setFile(const KURL &url)
{
	FileName_label->setText( url.fileName() );
	DirName_label->setText( url.directory(false,true) );

	QApplication::connect( &q, SIGNAL(receivedStdout(KProcess *, char *, int)),this,SLOT(slotOutputMessages(KProcess *, char *, int)));
	QApplication::connect( &q, SIGNAL(receivedStderr(KProcess *, char *, int)),this,SLOT(slotOutputMessages(KProcess *, char *, int)));

	if( url.fileName().right(7) == "tar.bz2" )
		{
		execute((  "bzip2 -cd " + DirName_label->text() + FileName_label->text() + " | tar xvf  -" ).latin1() );
		}
		else
		{
		execute((  "tar -tzf " + DirName_label->text() + FileName_label->text() ).latin1() );
		}
}

/** Set name of the file */
KURL ExtractDlg::getFile()
{
	KURL url;

	if( extractCurrent_radio->isChecked() )
		{
		url.setPath( DirName_label->text() + view_extractfiles->textLine(0) );
		}
		else
		{
		url.setPath( kurlrequest->url() + view_extractfiles->textLine(0) );
		}

	url.setFileName("configure");
	return url;
}

/** Check radio compress */
void ExtractDlg::slotExtractCheck()
{
	if( extractDir_radio->isChecked() )
	{		if( !kurlrequest->isEnabled() ) kurlrequest->setEnabled(true);		}
	else
	{		if( kurlrequest->isEnabled() ) kurlrequest->setEnabled(false);		}
}

/** Extract in current dir */
bool ExtractDlg::isCurrentDir()
{
	return extractCurrent_radio->isChecked();
}

/** Extract in personal directory */
QString ExtractDlg::extractDir()
{
	return kurlrequest->url();
}

/** Execute a command */
void ExtractDlg::execute(const char *file)
{
	QString Comm_Args;
	QString SFile = file;
	int buflen = SFile.length();
	
  q.clearArguments();
	streamLine = "";
	for( int i = 0; i < buflen; i++ )
	{
	if( *file == ' ' )
		{
		q << Comm_Args;
		Comm_Args = "";
		} else {
		Comm_Args += *file;
		}
	++file;
	}
	q << Comm_Args;

  q.start( KProcess::Block, KProcess::AllOutput );
}

/** No descriptions */
void ExtractDlg::slotOutputMessages(KProcess*,char *buffer,int buflen)
{
	for( int i = 0; i < buflen; i++ )
	{
	if( *buffer == '\n' )
		{
		view_extractfiles->insertLine( streamLine );
//		view_extractfiles->setCursorPosition(view_extractfiles->numLines(),0);
		streamLine = "";
		} else {
		streamLine += *buffer;
		}
	++buffer;
	}
}

/** No descriptions */
void ExtractDlg::slotFinishAction()
{
	KMessageBox::information(this, i18n("The extract process has finished"), i18n("Information"), QString::null, false);
}
