/***************************************************************************
                          kconfigure.cpp  -  description
                             -------------------
    begin                : sb abr 28 16:26:37 CEST 2001
    copyright            : (C) 2001 by Javier Campos
    email                : javi_cms@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kconfigure.h"
#include "kconfigure.moc"

#include "extractdlg.h"
#include "makecvs.h"

#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kurl.h>

#include <qfile.h>

Kconfigure::Kconfigure(QWidget *parent, const char *name, bool modal, WFlags f) : KWizard(parent, name, modal, f)
{
	(this->cancelButton())->setText(i18n("&Exit"));
	this->setIcon( kapp->miniIcon() );
}

Kconfigure::~Kconfigure()
{
	if( isConfigure ) delete page_conf;
	delete page_make;
	delete page_run;
	delete page_view;
}

/** Configure options page */
void Kconfigure::setPageView()
{
	page_view = new PageView(this, "page_view");
	page_view->init( urlProyect );
	if( !page_view->foundFiles() ) { delete page_view; return; }
	addPage( page_view, i18n( "Information" ) );
	setHelpEnabled ( page_view, false );
}

/** Configure options page */
void Kconfigure::setPageConfigure()
{
	page_conf = new PageConfigure(this, "page_conf");
	page_conf->init( urlProyect );
	addPage( page_conf, i18n( "Configure options" ) );
	setHelpEnabled ( page_conf, false );
}

/** Make options page */
void Kconfigure::setPageMake()
{
	page_make = new PageMake(this, "page_conf");
	addPage( page_make, i18n( "Makefile options" ) );
	setHelpEnabled ( page_make, false );
}

/** No descriptions */
void Kconfigure::setPageRun()
{
	page_run = new PageRun(this, "page_conf");
	page_run->setPath( urlProyect );
	addPage( page_run, i18n( "Final process" ) );
	setHelpEnabled ( page_run, false );
	setFinishEnabled ( page_run, false );
}

/** Show page operations */
void Kconfigure::showPage( QWidget *page)
{
    QWizard::showPage(page);

    if ( page == page_run )
		{
		  KConfig *config = KGlobal::config();
			config->setGroup("General");
			config->writeEntry("Geometry", this->size() );
			config->sync();
			page_make->configWrite();
			page_run->getMakeArgs( page_make->getArguments() );
			page_run->getInstallUser( page_make->getArgumentsInstall() );
			page_run->getInstallArgs( page_make->getArgumentsInstallMode() );
			if( isConfigure == true)
				{
				page_conf->configWrite();
				page_run->getConfigureArgs( page_conf->getArguments() );
				page_run->startLevel( 0 );
				}
				else
				{ page_run->startLevel( 1 ); }
    }
}

/** Open url */
bool Kconfigure::openURL( const KURL &url)
{

	if( url.fileName().right(6) == "tar.gz" || url.fileName().right(4) == ".tgz" || url.fileName().right(7) == "tar.bz2" )
		{
		if( !openExtractURL( url ) ) return false;
		}
		else
		{
		urlProyect = url;
		}

	this->setCaption("Kconfigure - [ " + urlProyect.url() + "]" );

 	if( url.fileName() == "Makefile.cvs" || url.fileName() == "Makefile.dist" )
		{
		MakeCVS cvs;
		cvs.setPath( urlProyect );
		cvs.setMakefile( url.fileName() );
		cvs.exec();
		}

	if	(
		!QFile::QFile(urlProyect.directory(false,true)+"configure").exists() &&
		!QFile::QFile(urlProyect.directory(false,true)+"Makefile").exists()
			)
  	{
		KMessageBox::error(0, i18n("SORRY !\n\nI cannot find any Makefile or Configure file.\n"), i18n("I cannot open proyect !"), false);
		return false;
		}

	setPageView();

	
	if( QFile::QFile(urlProyect.directory(false,true)+"configure").exists() )
		{
		isConfigure = true;
		setPageConfigure();
		}

	setPageMake();
	setPageRun();
	configRead();
	return true;
}

/** Open an extract file */
bool Kconfigure::openExtractURL( const KURL &url)
{
  ExtractDlg tarOptions;
	tarOptions.setFile( url );
	if(	tarOptions.exec() )
		{
		urlProyect = tarOptions.getFile();
		return true;
		}
		else
		{
		return false;
		}
}

/** Open new Documents */
bool Kconfigure::openNew()
{
  KURL url = KFileDialog::getOpenURL( QString::null, "configure Makefile Makefile.cvs Makefile.dist *.tar.gz *.tgz *.tar.bz2|All files supported\nconfigure|configure\nMakefile|Makefile\n*.tar.gz *.tgz *.tar.bz2|Source compress (tar.gz tar.bz2)", 0,  i18n("Open configure file") );
	if( url.isEmpty() )
		return false;

  return openURL( url );	
}

/** Config read */
void Kconfigure::configRead()
{
	KConfig *config = KGlobal::config();

	config->setGroup("General");
	QSize *defaultSize = new QSize(573,492);
	this->resize( config->readSizeEntry("Geometry", defaultSize ) );
}
