/***************************************************************************
                          makecvs.cpp  -  description
                             -------------------
    begin                : Thu Aug 30 2001
    copyright            : (C) 2001 by Javier Campos
    email                : javi_cms@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "makecvs.h"
#include "makecvs.moc"

#include <klocale.h>
#include <kstddirs.h>
#include <kmessagebox.h>
#include <knotifyclient.h>
#include <kconfig.h>

#include <qframe.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qscrollview.h>
#include <qvbox.h>


MakeCVS::MakeCVS( QWidget *parent, const char *name ) : KDialogBase(Swallow, i18n("CVS Options"), Ok|Cancel, Ok, parent, name, true, true)
{
	connect( &q, SIGNAL(receivedStdout(KProcess *, char *, int)),this,SLOT(slotOutputMessages(KProcess *, char *, int)));
	connect( &q, SIGNAL(receivedStderr(KProcess *, char *, int)),this,SLOT(slotOutputMessages(KProcess *, char *, int)));
	connect( &q, SIGNAL(processExited(KProcess *)),this, SLOT(slotFinishAction()) );

  KConfig *config = KGlobal::config();
	config->setGroup("Makefile CVS");
	QSize *defaultSize = new QSize(500,400);
	this->resize( config->readSizeEntry("Geometry", defaultSize ) );

	initGUI();

	this->enableButtonOK(false);
}

MakeCVS::~MakeCVS()
{
}

/** Init graphical interface */
void MakeCVS::initGUI()
{
		QWidget *mainWidget = new QWidget(this);

		QHBoxLayout *pageLayout_4 = new QHBoxLayout( mainWidget );
    pageLayout_4->setSpacing( 6 );

    QFrame *Frame14 = new QFrame( mainWidget, "Frame14" );
    Frame14->setMinimumSize( QSize( 125, 0 ) );
    Frame14->setMaximumSize( QSize( 125, 32767 ) );
		Frame14->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame.png")) );
    Frame14->setFrameShape( QFrame::Box );
    Frame14->setFrameShadow( QFrame::Plain );

    QVBoxLayout *Frame14Layout = new QVBoxLayout( Frame14 );
    Frame14Layout->setSpacing( 6 );
    Frame14Layout->setMargin( 11 );

    QLabel *pixmap_run = new QLabel( Frame14, "pixmap_run" );
    pixmap_run->setMinimumSize( QSize( 48, 48 ) );
    pixmap_run->setMaximumSize( QSize( 48, 48 ) );
		pixmap_run->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame2.png")) );
		pixmap_run->setPixmap( locate( "icon", "hicolor/48x48/apps/kconfigure.png" ) );
    Frame14Layout->addWidget( pixmap_run );

    QSpacerItem* spacer_10 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    Frame14Layout->addItem( spacer_10 );

    label_info = new QLabel( Frame14, "label_info" );
    label_info->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame.png")) );
		label_info->setText( i18n( "<p><b>CVS Detected</b></p><p>Used to create the configure and Makefiles for compile.</p>" ) );
    Frame14Layout->addWidget( label_info );

    QSpacerItem* spacer_11 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    Frame14Layout->addItem( spacer_11 );
    pageLayout_4->addWidget( Frame14 );

    QVBoxLayout *Layout14 = new QVBoxLayout;
    Layout14->setSpacing( 6 );
    Layout14->setMargin( 0 );

    QHBoxLayout *Layout13 = new QHBoxLayout;
    Layout13->setSpacing( 6 );
    Layout13->setMargin( 0 );

    QLabel *label_runstatus = new QLabel( mainWidget, "label_runstatus" );
    label_runstatus->setText( i18n( "Status :" ) );
    Layout13->addWidget( label_runstatus );

    label_runstatusinfo = new QLabel( mainWidget, "label_runstatusinfo" );
		label_runstatusinfo->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)7, (QSizePolicy::SizeType)5, label_runstatusinfo->sizePolicy().hasHeightForWidth() ) );
    label_runstatusinfo->setText( i18n( "<b>Stop</b>" ) );
    Layout13->addWidget( label_runstatusinfo );

		pixmap_runstatus = new KAnimWidget( QString("kde"), 22, mainWidget, "animWidget");
    Layout13->addWidget( pixmap_runstatus );
    Layout14->addLayout( Layout13 );

    view_status = new QMultiLineEdit( mainWidget, "view_status" );
		view_status->setWordWrap( QMultiLineEdit::NoWrap );
    Layout14->addWidget( view_status );

    QGroupBox *GroupBox21 = new QGroupBox( mainWidget, "GroupBox21" );
    GroupBox21->setTitle( i18n( "Actions" ) );
    GroupBox21->setColumnLayout(0, Qt::Vertical );
    GroupBox21->layout()->setSpacing( 0 );
    GroupBox21->layout()->setMargin( 0 );
    QHBoxLayout *GroupBox21Layout = new QHBoxLayout( GroupBox21->layout() );
    GroupBox21Layout->setAlignment( Qt::AlignTop );
    GroupBox21Layout->setSpacing( 6 );
    GroupBox21Layout->setMargin( 11 );

    button_runstart = new QPushButton( GroupBox21, "button_runstart" );
    button_runstart->setText( i18n( "&Start" ) );
		connect( button_runstart, SIGNAL( clicked() ), this, SLOT( slotStart() ) );
    GroupBox21Layout->addWidget( button_runstart );

    button_runstop = new QPushButton( GroupBox21, "button_runstop" );
    button_runstop->setText( i18n( "Stop" ) );
		connect( button_runstop, SIGNAL( clicked() ), this, SLOT( slotStop() ) );
    GroupBox21Layout->addWidget( button_runstop );

    Layout14->addWidget( GroupBox21 );
    pageLayout_4->addLayout( Layout14 );

		this->setMainWidget( mainWidget );
}

/** Execute a command */
void MakeCVS::execute( const char *command)
{
	QString Comm_Args;
	QString SFile = command;
	int buflen = SFile.length();

  if(q.isRunning()==true){
		KMessageBox::error(0, i18n("SORRY !\n\nWait until the process finish.\n"), i18n("I cannot do this !"), false);
		return;
		}
  q.clearArguments();
	// and identify after space like arguments
	for( int i = 0; i < buflen; i++ )
	{
	if( *command == ' ' )
		{
		q << Comm_Args;
		Comm_Args = "";
		} else {
		Comm_Args += *command;
		}
	++command;
	}
	q << Comm_Args;

	pixmap_runstatus->start();
  q.start( KProcess::NotifyOnExit, KProcess::AllOutput );
}

/** Start action */
void MakeCVS::slotStart()
{
	this->enableButtonCancel(false);
	label_runstatusinfo->setText( i18n("<b>Running...</b>") );
	execute(( "make -f "+ArgumentsCVS ).latin1());
	
}

/** Stop action */
void MakeCVS::slotStop()
{
	q.kill();
	this->enableButtonCancel(true);
}

/** Finish action */
void MakeCVS::slotFinishAction()
{
	pixmap_runstatus->stop();
	label_runstatusinfo->setText( i18n("<b>Stop</b>") );

	if( q.normalExit() && q.exitStatus()!=0 )
	{
	KMessageBox::error(0, i18n("ERROR !\n\nIt does not have a valid exit status."), i18n("Error"), false);
	this->enableButtonOK(false);
	this->enableButtonCancel(true);
	return;
	}

	if( q.normalExit() && q.exitStatus()==0 )
	{
			KNotifyClient::event("finish", i18n("The process has finished") );
			this->enableButtonOK(true);
			this->enableButtonCancel(false);
	}
	
}

/** Set path */
void MakeCVS::setPath( const KURL &url)
{
	qDir.setCurrent( url.directory(false,true) );
}

/** MakeFile */
void MakeCVS::setMakefile( const QString &file )
{
	ArgumentsCVS = file;
}

/** Output process */
void MakeCVS::slotOutputMessages(KProcess*,char *buffer,int buflen)
{
	for( int i = 0; i < buflen; i++ )
	{
	if( *buffer == '\n' )
		{
		view_status->insertLine( streamLine );
		view_status->setCursorPosition(view_status->numLines(),0);
		streamLine = "";
		} else {
		streamLine += *buffer;
		}
	++buffer;
	}	
}

/** Slot Ok */
void MakeCVS::slotOk()
{
  KConfig *config = KGlobal::config();
	config->setGroup("Makefile CVS");
	config->writeEntry( "Geometry", this->size() );
  config->sync();
	this->accept();
}
