/***************************************************************************
                          pageconfigure.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2001 by Javier Campos
    email                : javi_cms@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pageconfigure.h"
#include "pageconfigure.moc"
#include "predefined.h"

#include <klocale.h>
#include <kconfig.h>
#include <klineeditdlg.h>
#include <kstddirs.h>

#include <qpixmap.h>
#include <qbuttongroup.h>
#include <qframe.h>
#include <qlayout.h>
#include <qlineedit.h>

PageConfigure::PageConfigure(QWidget *parent, const char *name ) : QWidget(parent,name)
{
	initGUI();
	pred_conf = new predefined( locateLocal("appdata","predConf.ini") );
	combo_pred_configure->insertStringList( pred_conf->getGroups() );
	configRead();
}

PageConfigure::~PageConfigure()
{
  delete combo_pred_configure;
	delete button_pred_confsave;
  delete button_pred_confremove;
  delete view_confoptions;
  delete radio_conf_auto;
	delete radio_conf_manual;
  delete edit_conf_manual;
  delete check_conf_no;
  delete label_conf_options;
  delete pred_conf;
}

/** Init graphical interface */
void PageConfigure::initGUI()
{
		QHBoxLayout *pageLayout_2 = new QHBoxLayout( this );
    pageLayout_2->setSpacing( 6 );

    QFrame *Frame3_2 = new QFrame( this, "Frame3_2" );
    Frame3_2->setMinimumSize( QSize( 125, 0 ) );
    Frame3_2->setMaximumSize( QSize( 125, 32767 ) );
		Frame3_2->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame.png") ) );
    Frame3_2->setFrameShape( QFrame::Box );
    Frame3_2->setFrameShadow( QFrame::Plain );
    QVBoxLayout *Frame3_2Layout = new QVBoxLayout( Frame3_2 );
    Frame3_2Layout->setSpacing( 6 );
    Frame3_2Layout->setMargin( 11 );

    QLabel *pixmap_configure = new QLabel( Frame3_2, "pixmap_configure" );
    pixmap_configure->setMinimumSize( QSize( 48, 48 ) );
    pixmap_configure->setMaximumSize( QSize( 48, 48 ) );
		pixmap_configure->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame2.png") ) );
    pixmap_configure->setPixmap( locate( "icon", "hicolor/48x48/mimetypes/configure.png" ) );
    Frame3_2Layout->addWidget( pixmap_configure );
    QSpacerItem* spacer_4 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    Frame3_2Layout->addItem( spacer_4 );

    QLabel *label_configure = new QLabel( Frame3_2, "label_configure" );
		label_configure->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame.png") ) );
    label_configure->setText( i18n( "<p>It's <b>not necessary</b> to select options.</p><p/><p>The `configure' shell script attempts to guess correct values for various system-dependent variables used during compilation.</p>" ) );
    Frame3_2Layout->addWidget( label_configure );
    QSpacerItem* spacer_5 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    Frame3_2Layout->addItem( spacer_5 );
    pageLayout_2->addWidget( Frame3_2 );

    QVBoxLayout *Layout6 = new QVBoxLayout;
    Layout6->setSpacing( 6 );
    Layout6->setMargin( 0 );

    QGroupBox *GroupBox1 = new QGroupBox( this, "GroupBox1" );
    GroupBox1->setTitle( i18n( "Predefined options" ) );
    GroupBox1->setColumnLayout(0, Qt::Vertical );
    GroupBox1->layout()->setSpacing( 0 );
    GroupBox1->layout()->setMargin( 0 );
    QHBoxLayout *GroupBox1Layout = new QHBoxLayout( GroupBox1->layout() );
    GroupBox1Layout->setAlignment( Qt::AlignTop );
    GroupBox1Layout->setSpacing( 6 );
    GroupBox1Layout->setMargin( 11 );

    combo_pred_configure = new QComboBox( FALSE, GroupBox1, "combo_pred_configure" );
		connect( combo_pred_configure, SIGNAL( activated(int) ), this, SLOT( slotPredSelect(int) ) );
    GroupBox1Layout->addWidget( combo_pred_configure );

    button_pred_confsave = new QPushButton( GroupBox1, "button_pred_confsave" );
    button_pred_confsave->setText( i18n( "&Save" ) );
		connect( button_pred_confsave, SIGNAL( clicked() ), this, SLOT( slotPredSave() ) );
    GroupBox1Layout->addWidget( button_pred_confsave );

    button_pred_confremove = new QPushButton( GroupBox1, "button_pred_confremove" );
    button_pred_confremove->setText( i18n( "&Remove" ) );
		connect( button_pred_confremove, SIGNAL( clicked() ), this, SLOT( slotPredRemove() ) );
    GroupBox1Layout->addWidget( button_pred_confremove );
    Layout6->addWidget( GroupBox1 );

    QButtonGroup *GroupBox3 = new QButtonGroup( this, "GroupBox3" );
    GroupBox3->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)7, GroupBox3->sizePolicy().hasHeightForWidth() ) );
    GroupBox3->setTitle( i18n( "Options" ) );
    GroupBox3->setColumnLayout(0, Qt::Vertical );
    GroupBox3->layout()->setSpacing( 0 );
    GroupBox3->layout()->setMargin( 0 );
    QVBoxLayout *GroupBox3Layout = new QVBoxLayout( GroupBox3->layout() );
    GroupBox3Layout->setAlignment( Qt::AlignTop );
    GroupBox3Layout->setSpacing( 6 );
    GroupBox3Layout->setMargin( 11 );

    view_confoptions = new KListView( GroupBox3, "KListView2" );
    view_confoptions->addColumn( i18n( "Option" ) );
    view_confoptions->addColumn( i18n( "Value" ) );
    view_confoptions->addColumn( i18n( "Description" ) );
		view_confoptions->setColumnWidthMode( 0, QListView::Manual );
		view_confoptions->setColumnWidthMode( 1, QListView::Manual );
		view_confoptions->setColumnWidthMode( 2, QListView::Manual );
		view_confoptions->setAllColumnsShowFocus ( true );
		connect( view_confoptions, SIGNAL( executed(QListViewItem *) ), this, SLOT( slotItemModify(QListViewItem *) ) );
    GroupBox3Layout->addWidget( view_confoptions );

    radio_conf_auto = new QRadioButton( GroupBox3, "radio_conf_auto" );
    radio_conf_auto->setText( i18n( "Automatic" ) );
		connect( radio_conf_auto, SIGNAL( clicked() ), this, SLOT( slotRadioAutomatic() ) );
		radio_conf_auto->setChecked( true );
    GroupBox3Layout->addWidget( radio_conf_auto );

		QHBoxLayout *Layout13 = new QHBoxLayout;
    Layout13->setSpacing( 6 );
    Layout13->setMargin( 0 );
    radio_conf_manual = new QRadioButton( GroupBox3, "radio_conf_manual" );
    radio_conf_manual->setText( i18n( "Manual" ) );
		connect( radio_conf_manual, SIGNAL( clicked() ), this, SLOT( slotRadioAutomatic() ) );
    Layout13->addWidget( radio_conf_manual );
		edit_conf_manual = new QLineEdit( GroupBox3, "edit_conf_manual" );
		edit_conf_manual->setEnabled( false );
		Layout13->addWidget( edit_conf_manual );
		GroupBox3Layout->addLayout( Layout13 );

    QFrame *Frame8 = new QFrame( GroupBox3, "Frame8" );
    Frame8->setFrameShape( QFrame::HLine );
    Frame8->setFrameShadow( QFrame::Sunken );
    GroupBox3Layout->addWidget( Frame8 );

    QLabel *label_configure2 = new QLabel( GroupBox3, "label_configure2" );
    label_configure2->setText( i18n( "Current options :" ) );
    GroupBox3Layout->addWidget( label_configure2 );

    label_conf_options = new QLabel( GroupBox3, "label_conf_options" );
    label_conf_options->setAlignment( int( QLabel::WordBreak | QLabel::AlignVCenter | QLabel::AlignLeft ) );
    label_conf_options->setText( i18n("<No aditional arguments>") );
    GroupBox3Layout->addWidget( label_conf_options );
    Layout6->addWidget( GroupBox3 );
    pageLayout_2->addLayout( Layout6 );
}

/** Init configure help */
void PageConfigure::init( const KURL &url )
{
	KConfig *config = KGlobal::config();

	qDir.setCurrent( url.directory(false,true) );

	q.clearArguments();
	q << "./configure" << "--help";
	QApplication::connect( &q, SIGNAL(receivedStdout(KProcess *, char *, int)),this,SLOT(slotOutputMessages(KProcess *, char *, int)));
  q.start( KProcess::Block, KProcess::Stdout );

	config->setGroup("Configure Options");
	combo_pred_configure->setCurrentItem( config->readNumEntry("Last predefined", 0 ) );
	slotPredSelect( combo_pred_configure->currentItem() );
}

/** Output configure help */
void PageConfigure::slotOutputMessages(KProcess*,char*buffer,int buflen)
{
	for( int i = 0; i < buflen; i++ )
	{
	if( *buffer == '\n' )
		{
		addItem();
		streamLine = "";
		} else {
		streamLine += *buffer;
		}
	++buffer;
	}
}

/** Add item to klist */
void PageConfigure::addItem()
{
	int start_pos;
	int finish_opt;

	if( streamLine.right(1) == ":" ) return;
	if( streamLine.find("--help",0,false)>-1 || streamLine.find("--quiet",0,false)>-1 || streamLine.find("--version",0,false)>-1 ) return;

	if( (start_pos = streamLine.find("--")) > -1 )
		{
		finish_opt = streamLine.find(" ", start_pos);
		(void) new QListViewItem( view_confoptions, streamLine.mid( start_pos+2, finish_opt-2-start_pos ), "", ( streamLine.mid(finish_opt, streamLine.length()-finish_opt).stripWhiteSpace() ) );
		}
}

/** No descriptions */
void PageConfigure::slotItemModify(QListViewItem *item)
{
	if( (	  item->text(0).find("enable")>-1
			||  item->text(0).find("disable")>-1
			||  item->text(0)=="with-gnu-ld"
			||  item->text(0)=="with-pic"
			||  item->text(0)=="no-create"
			) && (item->text(0).find("FEATURE",0,false)==-1 && item->text(0).find("OPTS",0,false)==-1) && radio_conf_auto->isChecked() )
		{
			if( item->text(1) == i18n("Activate") )
				item->setText( 1, "" );
				else
				item->setText( 1, i18n("Activate") );
		}
		else
		{
		KLineEditDlg argdlg( "<p><b>"+item->text(0)+"</b></p><p>"+i18n("Description : ")+item->text(2)+"</p>", item->text(1), this);
		if(argdlg.exec())
			{
			if( argdlg.text() == QString::null )
					item->setText( 1, "" );
				else
					item->setText( 1, (argdlg.text()).stripWhiteSpace() );
			}
		}
	setArguments();
}

/** Print the arguments in the label */
void PageConfigure::setArguments()
{
	QString arguments="";

	QListViewItem *item = view_confoptions->firstChild();
    while( item )
		{
			if( item->text(1).stripWhiteSpace() != "" )
			{
			QString Soption = getValidArgument(item->text(0));
	
			if( item->text(1)==i18n("Activate") )
				{ arguments += "--" + Soption + " "; }
				else if( item->text(0).find("FEATURE")>-1 )
					{ arguments += "--" + Soption + item->text(1) + " "; }
					else
					{ arguments += "--" + Soption + "=" + item->text(1) + " "; }
			}

		item = item->nextSibling();

    }

	if( arguments.isEmpty() )
		label_conf_options->setText( i18n("<No aditional arguments>") );
		else
		label_conf_options->setText( arguments );

}

/** Fille predefined values */
void PageConfigure::PredefinedFill()
{

	clearData();

	while( !pred_conf->EntryEOF() )
	{
	 	QListViewItem *item = view_confoptions->firstChild();
    while( item )
		{
			if( getValidArgument(item->text(0)) == pred_conf->key() ) { item->setText(1, pred_conf->data()); break; }
			item = item->nextSibling();
    }
	pred_conf->nextEntry();
	}
	setArguments();

	if( pred_conf->readBoolEntry( "isManual", false ) )
		{
		radio_conf_manual->setChecked(true);
		slotRadioAutomatic();
		edit_conf_manual->setText( pred_conf->readEntry( "ManualArguments", "" ) );
		}
}

/** No descriptions */
void PageConfigure::slotPredSelect( int key )
{
	if( key==0 ) { clearData(); setArguments(); return; }

	pred_conf->selectGroup( combo_pred_configure->text(key) );
	PredefinedFill();
}

/** Get a valid argument */
QString PageConfigure::getValidArgument(const QString &argument)
{
	QString Soption = "";
	int SoptionLen;

	if( (SoptionLen = argument.find("[")) > -1 )
		{ Soption = argument.left(SoptionLen); }
		else 	if( (SoptionLen = argument.find("=")) > -1 )
			{ Soption = argument.left(SoptionLen); }
			else 	if( (SoptionLen = argument.find("FEATURE")) > -1 )
				{ Soption = argument.left(SoptionLen); }
				else { Soption = argument; }

	return Soption;
}

/** No descriptions */
void PageConfigure::slotPredSave()
{
		QString lineText="";

		if( combo_pred_configure->currentItem()>0 ) lineText=combo_pred_configure->currentText();

		KLineEditDlg argdlg( i18n("Save current arguments"), lineText, this);
		if(argdlg.exec() && argdlg.text() != QString::null )
			{
				if( pred_conf->hasGroup(argdlg.text()) )
					{
					pred_conf->deleteGroup( argdlg.text() );
					}
					else
					{
					combo_pred_configure->insertItem( argdlg.text() );
					combo_pred_configure->setCurrentItem( combo_pred_configure->count()-1 );
					}
				PredefinedSave(argdlg.text());
			}
}

/** No descriptions */
void PageConfigure::slotPredRemove()
{
	int Current = combo_pred_configure->currentItem();

	if( Current > 0 )
		{
		pred_conf->deleteGroup( combo_pred_configure->currentText() );
		pred_conf->sync();
		combo_pred_configure->removeItem( Current );
		combo_pred_configure->setCurrentItem( Current-1 );
		slotPredSelect( Current-1 );
		}
}

/** No descriptions */
void PageConfigure::PredefinedSave(const QString &group)
{
	pred_conf->addGroup( group );

	pred_conf->writeEntry( "isManual", radio_conf_manual->isChecked() );
	if( radio_conf_manual->isChecked() && !edit_conf_manual->text().isEmpty() )	pred_conf->writeEntry( "ManualArguments", edit_conf_manual->text() );

	QListViewItem *item = view_confoptions->firstChild();
    while( item )
		{
			if( item->text(1).stripWhiteSpace() != "" )
				pred_conf->addItem(getValidArgument(item->text(0)), item->text(1) );

		item = item->nextSibling();
    }

	pred_conf->sync();

}

/** Clear all data values */
void PageConfigure::clearData()
{
	// Clear all values
 	QListViewItem *item = view_confoptions->firstChild();
   while( item )
	{
		item->setText(1, "");
		item = item->nextSibling();
  }
	radio_conf_auto->setChecked(true);
	edit_conf_manual->setText("");
	slotRadioAutomatic();
}

/** Get arguments */
QString PageConfigure::getArguments()
{
	QString arguments = "";

	if( radio_conf_manual->isChecked() && !edit_conf_manual->text().isEmpty() )
		arguments += edit_conf_manual->text() + " ";

	if( label_conf_options->text() != i18n("<No aditional arguments>") )
		arguments += label_conf_options->text();

	return arguments;
}
/** Write config settings */
void PageConfigure::configWrite()
{
	KConfig *config = KGlobal::config();

	config->setGroup("Configure Options");
	config->writeEntry("Last predefined", combo_pred_configure->currentItem() );
	config->writeEntry("Column Option", view_confoptions->columnWidth(0) );
	config->writeEntry("Column Value", view_confoptions->columnWidth(1) );
	config->writeEntry("Column Description", view_confoptions->columnWidth(2) );
	config->sync();
}

/** Read config settings */
void PageConfigure::configRead()
{
	KConfig *config = KGlobal::config();

	config->setGroup("Configure Options");
	view_confoptions->setColumnWidth( 0, config->readNumEntry("Column Option", 139 ) );
	view_confoptions->setColumnWidth( 1, config->readNumEntry("Column Value", 79 ) );
	view_confoptions->setColumnWidth( 2, config->readNumEntry("Column Description", 167 ) );
}

/** Automatic radio */
void PageConfigure::slotRadioAutomatic()
{
	if( radio_conf_auto->isChecked() )
		{
			if( edit_conf_manual->isEnabled() ) edit_conf_manual->setEnabled( false );
		}	
		else
		{
			if( !edit_conf_manual->isEnabled() ) { edit_conf_manual->setEnabled( true ); edit_conf_manual->setFocus(); }
		}
}
