/***************************************************************************
                          pageconfigure.h  -  description
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2001 by Javier Campos
    email                : javi_cms@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PAGECONFIGURE_H
#define PAGECONFIGURE_H

#include "predefined.h"

#include <klistview.h>
#include <kprocess.h>
#include <kurl.h>

#include <qwidget.h>
#include <qdir.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>

/**Configure page options
  *@author Javier Campos
  */

class PageConfigure : public QWidget  {
   Q_OBJECT
public: 
	PageConfigure(QWidget *parent=0, const char *name=0);
	~PageConfigure();
  /** Init configure help */
  void init(const KURL &url);
  /** Get arguments */
  QString getArguments();
  /** Write config settings */
  void configWrite();
private: // Private methods
  /** Init graphical interface */
  void initGUI();
  /** Add item to klist */
  void addItem();
  /** Print the arguments in the label */
  void setArguments();
  /** Fille predefined values */
  void PredefinedFill();
  /** Save predefined values */
  void PredefinedSave(const QString &);
  /** Get a valid argument */
  QString getValidArgument(const QString &);
  /** Read config settings */
  void configRead();
  /** Clear all data values */
  void clearData();
private: // Private attributes
  /** Predefined configure options */
  QComboBox *combo_pred_configure;
  /** Predefined save */
  QPushButton *button_pred_confsave;
  /** Remove predefined configure options */
  QPushButton *button_pred_confremove;
  /** Options of configure */
  KListView *view_confoptions;
  /** Automatic selection control */
  QRadioButton *radio_conf_auto;
  /** Manual selection configure */
  QRadioButton *radio_conf_manual;
  /** Manual selection configure */
  QLineEdit *edit_conf_manual;
  /** No configure execute */
  QCheckBox *check_conf_no;
  /** Configure options selected */
  QLabel *label_conf_options;
	/** Main shell process of configure help */
	KShellProcess q;
  /** Temporal string data */
  QString streamLine;
  /** Predefined options */
  predefined *pred_conf;
  /** Current directory */
  QDir qDir;
private slots: // Private slots
  /** No descriptions */
  void slotItemModify(QListViewItem *item);
  /** Output configure help */
  void slotOutputMessages(KProcess*,char*buffer,int buflen);
  /** No descriptions */
  void slotPredSelect( int );
  /** No descriptions */
  void slotPredRemove();
  /** No descriptions */
  void slotPredSave();
  /** Automatic radio */
  void slotRadioAutomatic();
};

#endif
