/***************************************************************************
                          pagemake.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2001 by Javier Campos
    email                : javi_cms@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pagemake.h"
#include "pagemake.moc"

#include <kconfig.h>
#include <klocale.h>
#include <kstddirs.h>
#include <klineeditdlg.h>

#include <qbuttongroup.h>
#include <qframe.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qwhatsthis.h>

PageMake::PageMake(QWidget *parent, const char *name ) : QWidget(parent,name)
{
	initGUI();
	pred_make = new predefined( locateLocal("appdata","predMake.ini") );
	combo_pred_make->insertStringList( pred_make->getGroups() );
	configRead();
}

PageMake::~PageMake()
{
  delete combo_pred_make;
  delete button_pred_makesave;
  delete button_pred_makeremove;
  delete spin_makejob;
  delete edit_makeline;
  delete radio_installroot;
  delete radio_installuser;
  delete radio_installnormal;
  delete radio_installcheck;
  delete radio_installdebian;
  delete radio_installmanual;
  delete edit_installmanual;
  delete pred_make;
}

/** Init graphical interface */
void PageMake::initGUI()
{
    QHBoxLayout *pageLayout_3 = new QHBoxLayout( this );
    pageLayout_3->setSpacing( 6 );

    QFrame *Frame3_2_2 = new QFrame( this, "Frame3_2_2" );
    Frame3_2_2->setMinimumSize( QSize( 125, 0 ) );
    Frame3_2_2->setMaximumSize( QSize( 150, 32767 ) );
		Frame3_2_2->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame.png") ) );
    Frame3_2_2->setFrameShape( QFrame::Box );
    Frame3_2_2->setFrameShadow( QFrame::Plain );
    QVBoxLayout *Frame3_2_2Layout = new QVBoxLayout( Frame3_2_2 );
    Frame3_2_2Layout->setSpacing( 6 );
    Frame3_2_2Layout->setMargin( 11 );

    QLabel *pixmap_make = new QLabel( Frame3_2_2, "pixmap_make" );
    pixmap_make->setMinimumSize( QSize( 48, 48 ) );
    pixmap_make->setMaximumSize( QSize( 48, 48 ) );
		pixmap_make->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame2.png") ) );
    pixmap_make->setPixmap( locate( "icon", "hicolor/48x48/mimetypes/make.png" ) );
    pixmap_make->setScaledContents( TRUE );
    Frame3_2_2Layout->addWidget( pixmap_make );
    QSpacerItem* spacer_7 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    Frame3_2_2Layout->addItem( spacer_7 );

    QLabel *label_make = new QLabel( Frame3_2_2, "label_make" );
		label_make->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame.png") ) );
    label_make->setText( i18n( "<p><b>Make options</b></p><p>Run `make' with options to compile the package.<br>(Not necessary)</p><p/><p><b>Install options</b></p><p>Install the programs and any data files and documentation to a public path.</p>" ) );
    label_make->setAlignment( int( QLabel::WordBreak | QLabel::AlignVCenter | QLabel::AlignLeft ) );
    Frame3_2_2Layout->addWidget( label_make );
    QSpacerItem* spacer_8 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    Frame3_2_2Layout->addItem( spacer_8 );
    pageLayout_3->addWidget( Frame3_2_2 );

    QVBoxLayout *Layout16 = new QVBoxLayout;
    Layout16->setSpacing( 6 );
    Layout16->setMargin( 0 );

    QGroupBox *GroupBox1_2 = new QGroupBox( this, "GroupBox1_2" );
    GroupBox1_2->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, GroupBox1_2->sizePolicy().hasHeightForWidth() ) );
    GroupBox1_2->setTitle( i18n( "Predefined options" ) );
    GroupBox1_2->setColumnLayout(0, Qt::Vertical );
    GroupBox1_2->layout()->setSpacing( 0 );
    GroupBox1_2->layout()->setMargin( 0 );
    QHBoxLayout *GroupBox1_2Layout = new QHBoxLayout( GroupBox1_2->layout() );
    GroupBox1_2Layout->setAlignment( Qt::AlignTop );
    GroupBox1_2Layout->setSpacing( 6 );
    GroupBox1_2Layout->setMargin( 11 );

    combo_pred_make = new QComboBox( FALSE, GroupBox1_2, "combo_pred_make" );
		connect( combo_pred_make, SIGNAL( activated(int) ), this, SLOT( slotPredSelect(int) ) );
    GroupBox1_2Layout->addWidget( combo_pred_make );

    button_pred_makesave = new QPushButton( GroupBox1_2, "button_pred_makesave" );
    button_pred_makesave->setText( i18n( "&Save" ) );
		connect( button_pred_makesave, SIGNAL( clicked() ), this, SLOT( slotPredSave() ) );
    GroupBox1_2Layout->addWidget( button_pred_makesave );

    button_pred_makeremove = new QPushButton( GroupBox1_2, "button_pred_makeremove" );
    button_pred_makeremove->setText( i18n( "&Remove" ) );
		connect( button_pred_makeremove, SIGNAL( clicked() ), this, SLOT( slotPredRemove() ) );
    GroupBox1_2Layout->addWidget( button_pred_makeremove );
    Layout16->addWidget( GroupBox1_2 );

    QGroupBox *GroupBox8 = new QGroupBox( this, "GroupBox8" );
    GroupBox8->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)7, GroupBox8->sizePolicy().hasHeightForWidth() ) );
    GroupBox8->setTitle( i18n( "Makefile options" ) );
    GroupBox8->setColumnLayout(0, Qt::Vertical );
    GroupBox8->layout()->setSpacing( 0 );
    GroupBox8->layout()->setMargin( 0 );
    QVBoxLayout *GroupBox8Layout = new QVBoxLayout( GroupBox8->layout() );
    GroupBox8Layout->setAlignment( Qt::AlignTop );
    GroupBox8Layout->setSpacing( 6 );
    GroupBox8Layout->setMargin( 11 );

    QHBoxLayout *Layout9 = new QHBoxLayout;
    Layout9->setSpacing( 6 );
    Layout9->setMargin( 0 );

    QLabel *label_makejob = new QLabel( GroupBox8, "label_makejob" );
    label_makejob->setText( i18n( "Number of jobs :" ) );
    Layout9->addWidget( label_makejob );

    spin_makejob = new KIntSpinBox( GroupBox8, "spin_makejob" );
    spin_makejob->setValue( 1 );
    Layout9->addWidget( spin_makejob );
    QSpacerItem* spacer_9 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Layout9->addItem( spacer_9 );
    GroupBox8Layout->addLayout( Layout9 );

    QFrame *Frame13 = new QFrame( GroupBox8, "Frame13" );
    Frame13->setFrameShape( QFrame::HLine );
    Frame13->setFrameShadow( QFrame::Sunken );
    GroupBox8Layout->addWidget( Frame13 );

    QLabel *label_makeline = new QLabel( GroupBox8, "label_makeline" );
    label_makeline->setText( i18n( "Makefiles <FONT SIZE=-1>(ex. 'dep clean zImage modules' 'kvirc')</FONT>" ) );
    label_makeline->setAlignment( int( QLabel::WordBreak | QLabel::AlignVCenter | QLabel::AlignLeft ) );
    GroupBox8Layout->addWidget( label_makeline );

    edit_makeline = new QLineEdit( GroupBox8, "edit_makeline" );
    GroupBox8Layout->addWidget( edit_makeline );
    Layout16->addWidget( GroupBox8 );

    QButtonGroup *ButtonGroup11 = new QButtonGroup( this, "ButtonGroup11" );
    ButtonGroup11->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, ButtonGroup11->sizePolicy().hasHeightForWidth() ) );
    ButtonGroup11->setTitle( i18n( "User Install/Uninstall" ) );
    ButtonGroup11->setColumnLayout(0, Qt::Vertical );
    ButtonGroup11->layout()->setSpacing( 0 );
    ButtonGroup11->layout()->setMargin( 0 );
    QVBoxLayout *ButtonGroup11Layout = new QVBoxLayout( ButtonGroup11->layout() );
    ButtonGroup11Layout->setAlignment( Qt::AlignTop );
    ButtonGroup11Layout->setSpacing( 6 );
    ButtonGroup11Layout->setMargin( 11 );

    radio_installroot = new QRadioButton( ButtonGroup11, "radio_installroot" );
    radio_installroot->setText( i18n( "Install as root user (Default)" ) );
    radio_installroot->setChecked( TRUE );
    ButtonGroup11Layout->addWidget( radio_installroot );

    radio_installuser = new QRadioButton( ButtonGroup11, "radio_installuser" );
    radio_installuser->setText( i18n( "Install as current user (You need write permissions)" ) );
    ButtonGroup11Layout->addWidget( radio_installuser );

    QSpacerItem* spacer_43 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    ButtonGroup11Layout->addItem( spacer_43 );

    Layout16->addWidget( ButtonGroup11 );

    QButtonGroup *ButtonGroup21 = new QButtonGroup( this, "ButtonGroup21" );
    ButtonGroup21->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, ButtonGroup11->sizePolicy().hasHeightForWidth() ) );
    ButtonGroup21->setTitle( i18n( "Install/Uninstall options" ) );
    ButtonGroup21->setColumnLayout(0, Qt::Vertical );
    ButtonGroup21->layout()->setSpacing( 0 );
    ButtonGroup21->layout()->setMargin( 0 );
    QVBoxLayout *ButtonGroup21Layout = new QVBoxLayout( ButtonGroup21->layout() );
    ButtonGroup21Layout->setAlignment( Qt::AlignTop );
    ButtonGroup21Layout->setSpacing( 6 );
    ButtonGroup21Layout->setMargin( 11 );

    radio_installnormal = new QRadioButton( ButtonGroup21, "radio_installnormal" );
    radio_installnormal->setText( i18n( "Normal installation (make install)" ) );
		connect( radio_installnormal, SIGNAL( clicked() ), this, SLOT( slotRadioInstall() ) );
    ButtonGroup21Layout->addWidget( radio_installnormal );

    radio_installcheck = new QRadioButton( ButtonGroup21, "radio_installcheck" );
    radio_installcheck->setText( i18n( "Execute checkinstall (in progress...)" ) );
		radio_installcheck->setEnabled( false );
		connect( radio_installcheck, SIGNAL( clicked() ), this, SLOT( slotRadioInstall() ) );
    ButtonGroup21Layout->addWidget( radio_installcheck );

    radio_installdebian = new QRadioButton( ButtonGroup21, "radio_installdebian" );
    radio_installdebian->setText( i18n( "Make a Debian package (in progress...)" ) );
		radio_installdebian->setEnabled(false);
		connect( radio_installdebian, SIGNAL( clicked() ), this, SLOT( slotRadioInstall() ) );
    ButtonGroup21Layout->addWidget( radio_installdebian );

		QHBoxLayout *Layout13 = new QHBoxLayout;
    Layout13->setSpacing( 6 );
    Layout13->setMargin( 0 );
    radio_installmanual = new QRadioButton( ButtonGroup21, "radio_installmanual" );
    radio_installmanual->setText( i18n( "Install arguments" ) );
		connect( radio_installmanual, SIGNAL( clicked() ), this, SLOT( slotRadioInstall() ) );
    Layout13->addWidget( radio_installmanual );
		edit_installmanual = new QLineEdit( ButtonGroup21, "edit_install_manual" );
		edit_installmanual->setEnabled( false );
		Layout13->addWidget( edit_installmanual );
		ButtonGroup21Layout->addLayout( Layout13 );

    Layout16->addWidget( ButtonGroup21 );

    pageLayout_3->addLayout( Layout16 );
}

/** No descriptions */
void PageMake::slotPredSelect( int key )
{
	if( key==0 ) { fillDefault(); return; }

	pred_make->selectGroup( combo_pred_make->text(key) );
	fillPred();
}

/** No descriptions */
void PageMake::slotPredSave()
{
		QString lineText="";

		if( combo_pred_make->currentItem()>0 ) lineText=combo_pred_make->currentText();

		KLineEditDlg argdlg( i18n("Save current settings"), lineText, this);
		if(argdlg.exec() && argdlg.text() != QString::null )
			{
				if( pred_make->hasGroup(argdlg.text()) )
					{
					pred_make->deleteGroup( argdlg.text() );
					}
					else
					{
					combo_pred_make->insertItem( argdlg.text() );
					combo_pred_make->setCurrentItem( combo_pred_make->count()-1 );
					}
				PredefinedSave( argdlg.text() );
			}
}

/** No descriptions */
void PageMake::slotPredRemove()
{
	int Current = combo_pred_make->currentItem();

	if( Current > 0 )
		{
		pred_make->deleteGroup( combo_pred_make->currentText() );
		pred_make->sync();
		combo_pred_make->removeItem( Current );
		combo_pred_make->setCurrentItem( Current-1 );
		slotPredSelect( Current-1 );
		}
}

/** Fill default arguments */
void PageMake::fillDefault()
{
//	check_makedebug->setChecked(false);
//	check_makeerror->setChecked(false);
	edit_makeline->setText("");
	radio_installroot->setChecked(true);
	spin_makejob->setValue(1);
	radio_installnormal->setChecked(true);
}

/** Fill predefined options */
void PageMake::fillPred()
{
//	check_makedebug->setChecked( pred_make->readBoolEntry( "debug", false ) );
//	check_makeerror->setChecked( pred_make->readBoolEntry( "error", false ) );
	spin_makejob->setValue( pred_make->readNumEntry( "jobs", 1 ) );
	edit_makeline->setText( pred_make->readEntry( "line", "" ) );
	
	if( pred_make->readBoolEntry( "RootInstall", true ) )
		radio_installroot->setChecked(true);
		else
		radio_installuser->setChecked(true);

	if( pred_make->readEntry( "ModeInstall", "normal" ) == "normal" )
		{ radio_installnormal->setChecked(true); }
		else if( pred_make->readEntry( "ModeInstall" ) == "checkinstall"  )
			{ radio_installcheck->setChecked(true); }
			else if( pred_make->readEntry( "ModeInstall" ) == "debian" )
				{ radio_installdebian->setChecked(true); }
				else if( pred_make->readEntry( "ModeInstall" ) == "manual" )
					{ radio_installmanual->setChecked(true);
						edit_installmanual->setText( pred_make->readEntry( "ManualInstall" ) );
					}
}

/** Save predefined options */
void PageMake::PredefinedSave( const QString &group)
{
	pred_make->addGroup( group );
	
//  pred_make->writeEntry( "debug", check_makedebug->isChecked() );
//  pred_make->writeEntry( "error", check_makeerror->isChecked() );
	pred_make->writeEntry( "jobs", spin_makejob->value() );
  pred_make->writeEntry( "line", edit_makeline->text() );

	if( radio_installroot->isChecked() == true )
		pred_make->writeEntry("RootInstall", true);
		else
		pred_make->writeEntry("RootInstall", false);

	if( radio_installnormal->isChecked() )
		{ pred_make->writeEntry( "ModeInstall", "normal" ); }
		else if( radio_installcheck->isChecked() )
			{	pred_make->writeEntry( "ModeInstall", "check" ); }
			else if( radio_installdebian->isChecked() )
				{ pred_make->writeEntry( "ModeInstall", "debian" ); }
				else if( radio_installmanual->isChecked() )
					{ pred_make->writeEntry( "ModeInstall", "manual" );
						pred_make->writeEntry( "ManualInstall", edit_installmanual->text() );
					}

	pred_make->sync();
}

/** Return make arguments */
QString PageMake::getArguments()
{
	QString arguments("");

//	if( check_makedebug->isChecked() ) arguments += " -d";
//	if( check_makeerror->isChecked() ) arguments += " -k";
	if( spin_makejob->value()>1 ) arguments += " -j" + QString().setNum(spin_makejob->value());
	if( !(edit_makeline->text()).isEmpty() ) arguments += " " + edit_makeline->text();

	return arguments;
}

/** Return install arguments */
QString PageMake::getArgumentsInstall()
{
	if( radio_installroot->isChecked() == true )
		return "root";
		else
		return "user";
}

/** Return install arguments */
QString PageMake::getArgumentsInstallMode()
{
	if( radio_installcheck->isChecked() == true )
		return "check";
	if( radio_installmanual->isChecked() && !(edit_installmanual->text()).isEmpty() )
		return edit_installmanual->text();
	return "install";
}

/** Read config settings */
void PageMake::configRead()
{
	KConfig *config = KGlobal::config();

	config->setGroup("Makefile Options");
	combo_pred_make->setCurrentItem( config->readNumEntry("Last predefined", 0 ) );
	slotPredSelect( combo_pred_make->currentItem() );
}

/** Write config settings */
void PageMake::configWrite()
{
	KConfig *config = KGlobal::config();

	config->setGroup("Makefile Options");
	config->writeEntry("Last predefined", combo_pred_make->currentItem() );
}

/** No descriptions */
void PageMake::slotRadioInstall()
{
	if( radio_installmanual->isChecked() )
	{	if( !edit_installmanual->isEnabled() ) edit_installmanual->setEnabled(true); edit_installmanual->setFocus(); }
	else
	{	if( edit_installmanual->isEnabled() ) edit_installmanual->setEnabled(false); }
}
