/***************************************************************************
                          pageview.cpp  -  description
                             -------------------
    begin                : Thu May 3 2001
    copyright            : (C) 2001 by Javier Campos
    email                : javi_cms@terra.es
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pageview.h"
#include "pageview.moc"

#include <klocale.h>
#include <kstddirs.h>

#include <qframe.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qfile.h>
#include <qtextstream.h>

PageView::PageView(QWidget *parent, const char *name ) : QWidget(parent,name)
{
	initGUI();
}

PageView::~PageView()
{
	delete combo_files;
	delete view_files;
}

/** Init Graphical Interface */
void PageView::initGUI()
{
    QVBoxLayout *pageLayout_5 = new QVBoxLayout( this );
    pageLayout_5->setSpacing( 6 );
    pageLayout_5->setMargin( 0 );

    QFrame *Frame10 = new QFrame( this, "Frame10" );
    Frame10->setMinimumSize( QSize( 0, 0 ) );
    Frame10->setMaximumSize( QSize( 32767, 32767 ) );
		Frame10->setPalette( QPalette( QColor(255, 255, 255) ) );
    Frame10->setFrameShape( QFrame::Box );
    Frame10->setFrameShadow( QFrame::Plain );
    QHBoxLayout *Frame10Layout = new QHBoxLayout( Frame10 );
    Frame10Layout->setSpacing( 6 );
    Frame10Layout->setMargin( 1 );

    QFrame *Frame13_3 = new QFrame( Frame10, "Frame13_3" );
    Frame13_3->setMinimumSize( QSize( 125, 0 ) );
    Frame13_3->setMaximumSize( QSize( 125, 32767 ) );
		Frame13_3->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame.png") ) );
    Frame13_3->setFrameShape( QFrame::NoFrame );
    Frame13_3->setFrameShadow( QFrame::Plain );
    QVBoxLayout *Frame13_3Layout = new QVBoxLayout( Frame13_3 );
    Frame13_3Layout->setSpacing( 6 );
    Frame13_3Layout->setMargin( 11 );

    QLabel *PixmapLabel2 = new QLabel( Frame13_3, "PixmapLabel2" );
    PixmapLabel2->setMinimumSize( QSize( 48, 48 ) );
    PixmapLabel2->setMaximumSize( QSize( 48, 48 ) );
		PixmapLabel2->setBackgroundPixmap( QPixmap(locate("appdata","pics/bk_frame2.png") ) );
    PixmapLabel2->setPixmap( locate( "icon", "hicolor/48x48/mimetypes/info.png" ) );
    Frame13_3Layout->addWidget( PixmapLabel2 );
    Frame10Layout->addWidget( Frame13_3 );

    QSpacerItem* spacer_13 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    Frame10Layout->addItem( spacer_13 );

    QLabel *TextLabel3_2 = new QLabel( Frame10, "TextLabel3_2" );
		TextLabel3_2->setPalette( QPalette( QColor(255, 255, 255) ) );
    TextLabel3_2->setText( i18n( "Files : " ) );
    Frame10Layout->addWidget( TextLabel3_2 );

    combo_files = new QComboBox( FALSE, Frame10, "ComboBox4_2" );
		connect( combo_files, SIGNAL( activated(int) ), this, SLOT( slotView(int) ) );
    Frame10Layout->addWidget( combo_files );
    QSpacerItem* spacer_14 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum );
    Frame10Layout->addItem( spacer_14 );
    pageLayout_5->addWidget( Frame10 );

    QFrame *Frame12 = new QFrame( this, "Frame12" );
    Frame12->setFrameShape( QFrame::StyledPanel );
    Frame12->setFrameShadow( QFrame::Raised );
    QVBoxLayout *Frame12Layout = new QVBoxLayout( Frame12 );
    Frame12Layout->setSpacing( 6 );
    Frame12Layout->setMargin( 6 );

    view_files = new QTextView( Frame12, "view_files" );
    Frame12Layout->addWidget( view_files );
    pageLayout_5->addWidget( Frame12 );
}

/** Find files to view */
void PageView::findFiles()
{
	QStringList list;
	list << "README" << "Readme" << "readme";
	list << "INSTALL" << "Install" << "install";
	list << "CHANGELOG" << "ChangeLog" << "changelog";
	list << "CHANGES" << "Changes" << "changes";
	list << "AUTHORS" << "Authors" << "authors";
	list << "COMPILING" << "BUGS" << "BUGSLIST";
	list << "NEWS" << "FAQ" << "TODO";

	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
		{
		if( QFile::QFile( (*it).latin1() ).exists() ) combo_files->insertItem( (*it).latin1() );
		}
}

/** Set initial Path */
void PageView::init( const KURL &url )
{
	qDir.setCurrent( url.directory(false,true) );

	findFiles();
	if( combo_files->count() > 0 ) slotView(0);
}
/** Combo box */
void PageView::slotView( int index )
{
	QFile file( combo_files->text( index ) );
	if ( file.open(IO_ReadOnly) )
		{
	  QTextStream stream(&file);
		view_files->setText( stream.read() );
		file.close();
		}
}

/** Do I find files to read? */
bool PageView::foundFiles()
{
	if( combo_files->count() == 0 ) return false;
	return true;
}
